/* 

FILE:   scribacmd.c
HEADER: scribacmd.h

--GNU LGPL
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

TO_HEADER:

*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../../getopt.h"
#include "../../scriba.h"

#include "../../basext.h"

#if BCC32
char *_pgmptr;
#endif


int GetC(void *f){ return getc((FILE *)f); }

#ifdef _DEBUG
#define ERREXIT do{getchar();scriba_destroy(pProgram);exit(iError);}while(0)
#else
#define ERREXIT exit(iError)
#endif

#ifdef WIN32
main(int argc, char *argv[]){
#else
  char **_environ;
main(int argc, char *argv[], char *env[]){
#endif
  int iError;
  int iErrorCounter;
  int OptionIndex;
  pSbProgram pProgram;
  char *pszForcedConfigurationFileName;
  unsigned long fErrorFlags;
  char *szInputFile,
       *szOutputFile;
  char *optarg,opt;
  /* the maximal number of preprocessors that are applied in chain */
#define MAXPREPROC 100
  char *pszEPreproc[MAXPREPROC],*pszPreprocessedFileName;
  char *pszIPreproc[MAXPREPROC];
  int iEPreprocIndex,iIPreprocIndex;
  int giveusage,binarycode,nocache,iscgi,isCoutput;
  int execute;
#define FULL_PATH_BUFFER_LENGTH 256
  char CmdLinBuffer[FULL_PATH_BUFFER_LENGTH];
  extern int GlobalDebugDisplayFlag;

/*pSbData p,psb;
SbData RetVale;*/
#ifndef WIN32
  _environ = env;
#endif

#if BCC32
  _pgmptr = argv[0];
#endif

#ifdef _DEBUG
#define malloc testa_Alloc
#define free testa_Free
  testa_InitSegment();
#endif
  /* default values for command line options */
  szInputFile = NULL;
  szOutputFile = NULL;
  iEPreprocIndex = 0; /* no external preprocessor by default */
  iIPreprocIndex = 0; /* no internal preprocessor by default */
  pszPreprocessedFileName = NULL;
  *pszEPreproc = NULL; 
  *pszIPreproc = NULL;
  giveusage  = 0; /* assume the command line is correct, we need not display usage and stop */
  binarycode = 0; /* input is not binary by default */
  execute    = 0; /* do not execute by default after binary format save */
  nocache   = 0; /* we use cached code if it exists */
  OptionIndex = 0;
  iscgi = 0; /* by default this is not a cgi script, not HTTP/1.0 ... when error message is sent */
  isCoutput = 0;
  pszForcedConfigurationFileName = NULL;
  GlobalDebugDisplayFlag = 0;

  while( (opt = getoptt(argc, argv, "df:p:i:Ccnvebo:",&optarg,&OptionIndex)) != ':'){
    switch( opt ){
      case 'd' :
        GlobalDebugDisplayFlag = 1;
        break;
      case 'p' :
        if( iEPreprocIndex >= MAXPREPROC-1 )
          giveusage = 1;
        else{
          pszEPreproc[iEPreprocIndex ++ ] = optarg;
          pszEPreproc[iEPreprocIndex] = NULL;
          }
         break;
      case 'i' :
        if( iIPreprocIndex >= MAXPREPROC-1 )
          giveusage = 1;
        else{
          pszIPreproc[iIPreprocIndex ++ ] = optarg;
          pszIPreproc[iIPreprocIndex] = NULL;
          }
         break;
      case 'C' :
        if( isCoutput )giveusage = 1;
        isCoutput = 1;
        break;
      case 'c' :
        if( iscgi )giveusage = 1;
        iscgi = 1;
        break;
      case 'n' :
        if( nocache )giveusage = 1;
        nocache = 1;
        break;
      case 'e' :
        if( execute )giveusage = 1;
        execute = 1;
        break;
      case 'f' :
        if( pszForcedConfigurationFileName )giveusage = 1;
        pszForcedConfigurationFileName = optarg;
        break;
      case 'o' :
        if( szOutputFile || binarycode )giveusage = 1;
        szOutputFile = optarg;
        break;
      case 'b':
        if( szOutputFile || binarycode )giveusage = 1;
        binarycode =1;
        break;
      case 'v':
#define S fprintf(stderr,
#define E );
                   S "ScriptBasic v%ld.%ld\n",VERSION_HIGH,VERSION_LOW E
                   S "Variation >>%s<< build %ld\n",VARIATION,SCRIPTBASIC_BUILD E
                   S "Magic value %lu\n",build_MagicCode(NULL) E
                   S "Node size is %d\n", sizeof(cNODE) E
                   S "Extension interface version is %d\n",INTERFACE_VERSION E
                   S "Compilation: %s %s\n", __DATE__,__TIME__ E
#ifdef WIN32
                   S "Executable: %s\n", _pgmptr E
#endif
        exit(0);
      case '!' :
        giveusage = 1;
        break;
      case '?':
        if( szInputFile )giveusage = 1;
        szInputFile = optarg;
        CmdLinBuffer[0] = (char)0;
        while( OptionIndex < argc ){
          strcat(CmdLinBuffer,argv[OptionIndex++]);
          if( OptionIndex < argc )
            strcat(CmdLinBuffer," ");
          }
        goto CmdLineFinished;
      }
    }

CmdLineFinished:
  if( execute && binarycode )giveusage=1;
  if( isCoutput && !szOutputFile )giveusage=1;
  if( iIPreprocIndex )nocache = 1;
/* NOTE: that external preprocessor effects can be cached!
  if( iEPreprocIndex )nocache = 1;
   On the other hand internal preprocessor provided on the
   command-line (and not in the source 'preprocess' commands)
   is most probably a debugger or profiler, thus no caching
   should be done, because if we use the cache the program
   runs without having the preprocessor in effect.
*/
  if( szInputFile == NULL || giveusage ){
#define U(x) fprintf(stderr,"%s\n",(x));
    U("Usage: basic [options] program.bas")
    U("")
    U("options: -o file_name")
    U("            save binary format to file but don't execute")
    U("         -b file_name")
    U("            load binary format from file and execute")
    U("         -n")
    U("            do not use cache (no save, no load)")
    U("         -e")
    U("            execute after binary format was saved")
    U("         -v")
    U("            print version info and stop")
    U("         -c")
    U("            inform scriba that this is a CGI script.")
    U("         -C");
    U("            save C program output.");
    U("         -p preprocessor");
    U("            specify external preprocessor.");
    U("         -i preprocessor");
    U("            specify internal preprocessor.");
    U("         -f configurationfile");
    U("            specify configuration file");
    exit(1);
    }

  pProgram = scriba_new(malloc,free);
  if( pProgram == NULL ){
    iError = COMMAND_ERROR_MEMORY_LOW;
    ERREXIT;
    }

  scriba_LoadConfiguration(pProgram,pszForcedConfigurationFileName);

  iError = scriba_LoadInternalPreprocessor(pProgram,pszIPreproc);
  if( iError ){
    report_report(stderr,"",0,iError,REPORT_ERROR,&iErrorCounter,NULL,&fErrorFlags);
    ERREXIT;
    }

  if( iscgi )scriba_SetCgiFlag(pProgram);

  scriba_SetFileName(pProgram,szInputFile);

  if( (!nocache) && scriba_UseCacheFile(pProgram) == SCRIBA_ERROR_SUCCESS )binarycode = 1;
  if( binarycode || scriba_IsFileBinaryFormat(pProgram) ){
    if( (iError = scriba_LoadBinaryProgram(pProgram)) != 0 ){
      ERREXIT;
      }
    }else{
    if( iError=scriba_RunExternalPreprocessor(pProgram,pszEPreproc) ){
      report_report(stderr,"",0,iError,REPORT_ERROR,&iErrorCounter,NULL,&fErrorFlags);
      ERREXIT;
      }
    if( scriba_LoadSourceProgram(pProgram) )ERREXIT;
    if( szOutputFile ){
      if( isCoutput )
        scriba_SaveCCode(pProgram,szOutputFile);
      else
        scriba_SaveCode(pProgram,szOutputFile);
      if( !execute )exit(0);
      }
     if( ! nocache )scriba_SaveCacheFile(pProgram);
    }
/*
  {FILE *fb; fb = fopen("dumpcode.txt","w");
  build_pprint(pProgram->pBUILD,fb);fclose(fb);}
*/
/*#define PRGSTR "print \"hello\\n\"\n\
include \"test.bas\"\
"
  scriba_LoadProgramString(pProgram,PRGSTR,strlen(PRGSTR));
*/
  if( iError=scriba_Run(pProgram,CmdLinBuffer) ){
    report_report(stderr,"",0,iError,REPORT_ERROR,&iErrorCounter,NULL,&fErrorFlags);
    ERREXIT;
    }

/*
{
unsigned long i=0;
unsigned long min,max,act;
pSbData pRetVale;
  while( 1 ){
    i++;
    if( i%10000 == 0 ){
      testa_ReportLeak();
#define PRSEG(X)      alloc_GetStat(X,&min,&max,&act);printf("Segment %s is %d\n",#X,act);
      PRSEG(pProgram->pMEM);
      PRSEG(pProgram->pEXE->pMo);
      PRSEG(pProgram->pEXE->pMemorySegment);
      printf("kukk %d\n",i/10000);
      getchar();
      }
    pRetVale = scriba_NewSbData(pProgram);
    scriba_CallArgEx(pProgram,scriba_LookupFunctionByName(pProgram,"main::pred_1"),pRetVale,0,NULL);
    scriba_DestroySbData(pProgram,pRetVale);
    }
}

  printf("testvar type is %d\n",scriba_GetVariableType(pProgram,
  scriba_LookupVariableByName(pProgram,"main::testvar")));
  scriba_GetVariable(pProgram,
                     scriba_LookupVariableByName(pProgram,"main::testvar"),
                     &psb
    );
  printf("testvar from C is %s\n",scriba_GetString(pProgram,*psb));

  scriba_SetVariable(pProgram,scriba_LookupVariableByName(pProgram,"main::testvar"),
                     SBT_ZCHAR,1,2.2,"kukac",4);
  scriba_CallArgEx(pProgram,scriba_LookupFunctionByName(pProgram,"main::thatfunc"),&RetVale,4,p);
*/
  scriba_destroy(pProgram);

#ifdef _DEBUG
  testa_AssertLeak();
  if( ! iscgi ){
    printf("\nPress any key to continue...\n");
    getchar();
    }
#endif
  exit(iError);
  }

